
/*
 * GPIO Functions
 * [Jason Chou], 2012/04/30 - add LED Switch for OMNINAS D20
 * [Jason Chou], 2012/04/30 - add Power off for OMNINAS D20
 * GPIO Functions
 * [Walker Chen], 2010/05/14 - add LED Switch for Z1N511
 * [Walker Chen], 2010/05/14 - add Power off for Z1N511
 *
 * Disk Functions
 * [Walker Chen], 2010/05/21 - add "clrenv" Function
 * [Walker Chen], 2010/05/27 - add "qazwsx" Function
 *
 * NAND Functions
 * [walker Chen], 2010/07/14 - "clrenv" support for NAND
 * [walker Chen], 2010/07/14 - add "bubt" Function
 *
 * DAS Functions
 * [Walker Chen], 2010/08/10 - converts to DAS mode if vbus is high
 *
 */

#include <common.h>
#include <command.h>
#include <net.h>

#ifdef ENV_ON_NAND
#include <nand.h>
extern nand_info_t nand_info[CFG_MAX_NAND_DEVICE];
#endif //ENV_ON_NAND

#include <ide.h>
extern int is_device_present(int device_number);


extern int	uboot_is_das;
extern	int red_brightness;
extern	int green_brightness;
extern	int blue_brightness;
int load_leds_brightness( void );


inline void led_on( void ){

		*(volatile u32*)GPIO_2_OE  |=  ( 1 << LED_STATUS_B );
		*(volatile u32*)GPIO_2_OUT |= ( 1 << LED_STATUS_B );
}

inline void led_red( void ){


		*(volatile u32*)GPIO_2_OE  |= ( 1 << LED_STATUS_R);	// Set GPIOB Output
		*(volatile u32*)GPIO_2_OUT &= ~( 1 << LED_STATUS_B);
		*(volatile u32*)GPIO_2_OUT |= ( 1 << LED_STATUS_R);
}

inline void led_off( void ){

	*(volatile u32*)GPIO_2_OE  	|= ( 1 << LED_STATUS_B |1 << LED_STATUS_R);
	*(volatile u32*)GPIO_2_OUT  &= ~( 1 << LED_STATUS_B |1 << LED_STATUS_R);
}

inline void led_err( void ){
	extern int uboot_err;
	//uboot_err=1;
	uboot_err = !uboot_err;
	if ( uboot_err ){

	*(volatile u32*)GPIO_1_OE  	|=  (1<<LED_STATUS_R); // GPIO1-2 set to be output
	*(volatile u32*)GPIO_2_OUT &= ~( 1 << LED_STATUS_B);
	*(volatile u32*)GPIO_1_OUT  |=  (1<<LED_STATUS_R); // GPIO1-2 set to be hight
	
 	enable_interrupts();
	}
}

inline void led_state( void ){	
  	
	printf ("LED_HDDA_B is %s %s \n",
		   (*(volatile u32*)GPIO_1_OE&(1<<LED_HDDA_B))?"OUTPUT":"INPUT",
		   (*(volatile u32*)GPIO_1_PA&(1<<LED_HDDA_B))?"HIGHT":"LOW" );
    	
	printf ("Timer is 0x%04X \n", ((*(volatile ushort *)(CFG_TIMERBASE+4)) & 0xFFFFUL));
	
}

int cmd_led (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
//	int i;
//	printf("do CMD_LED argc:%d\n", argc);
//	for (i = 0; i < argc; i++) printf("argv[%d]:%s:\n", i, argv[i]);

	if (strcmp(argv[1],"1") == 0) {
		/*  LED blue */
		disable_interrupts();
		led_on();

	}else if (strcmp(argv[1],"0") == 0){
		/* LED off */
		disable_interrupts();
		led_off();

	}else if(strcmp(argv[1],"r") == 0){
		/*  LED red */
		led_red();

	}else if(strcmp(argv[1],"a") == 0){
		printf ("enable_interrupts\n");
		enable_interrupts();

	}else if(strcmp(argv[1],"z") == 0){
		printf ("disable_interrupts\n");
		disable_interrupts();

	}else if(strcmp(argv[1],"x") == 0){
		/* just for test */
		//load_leds_brightness();

	}else{
		//led_state();
	}
return (0);
}

U_BOOT_CMD(
	led,     2,     1,      cmd_led,
	"led\t- LED Switch\n",
	"[1/0] = LED ON/OFF \n"
	"[a/z] = Interrupts ON/OFF \n"
	"Walker's LED switch \n "
);

int cmd_poweroff (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	*(volatile u32*)GPIO_1_OUT  	|=  (1<<PWR_OFF); // GPIO1-9 set to be high
	*(volatile u32*)GPIO_1_SET_OE  	|=  (1<<PWR_OFF); // GPIO1-9 set to be output
	printf ("System is Shoutdown Now\n");

	return(0);
}

U_BOOT_CMD(
	poweroff,     2,     1,      cmd_poweroff,
	"poweroff- Power-OFF\n",
	"N/A \n"
	"Walker's Power-OFF Function \n "
);

int cmd_beep (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	*(volatile u32*)GPIO_2_SET_OE  	|=  (1<<BUZZER_PIN); // GPIO1-24 set to be output	
	*(volatile u32*)PWM_43_Value = 0x7F;		// GPIO1-24_pwm_duty set to 50%
	udelay(100000);		//delay 100msec
	*(volatile u32*)PWM_43_Value = 0x00;		// GPIO1-24_pwm_duty set to 0%
	udelay(100000);		//delay 100msec
	return(0);
}
U_BOOT_CMD(
	beep,     2,     1,      cmd_beep,
	"beep\t- make beep sound\n",
	"N/A \n"
	"Walker's beep Function \n "
);

int cmd_clrenv (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
#ifdef ENV_ON_SATA
    static char temp[CFG_ENV_SIZE];
    memset(temp,'\0',sizeof(temp));
    printf ("clear environment in disk\n");
    ide_write(0, CFG_ENV_DISK_SECTOR, CFG_ENV_SIZE/512,(u32*)&temp );
    ide_write(0, CFG_ENV_DISK_REDUNDANT_SECTOR, CFG_ENV_SIZE/512,(u32*)&temp);
#endif //ENV_ON_SATA

#ifdef ENV_ON_NAND
	printf ("clear environment in NAND\n");
	nand_erase(&nand_info[0], CFG_ENV_OFFSET, NAND_BLOCK_SIZE);
#endif //ENV_ON_NAND

return(0);
}

U_BOOT_CMD(
	clrenv,     2,     1,      cmd_clrenv,
	"clrenv\t- clear environment\n",
	"N/A \n"
	"Walker's Clear environment Function \n "
);

int cmd_qazwsx (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	static struct MBR{char 	code[446],
						 	part1[16],
						 	part2[16],
						 	part3[16],
						 	part4[16],
						 	signature[2];	}mbr_0,mbr_8 ;

	/* MBR debug
	ide_read(0, 0, 1, (ulong*)&mbr_0);
	printf("MBR_adr address :%x \n", &mbr_0 );
	printf("MBR signature is %x \n",*(u16*)mbr_0.signature );
	memcpy(mbr_0.part3,mbr_0.part1,16); //part1 cp to part3
	printf("part1 cp to part3 \n");
	*/

	if (!is_device_present(0)) return(1); //no disk

	if(strcmp(argv[1],"hide") == 0){

		/* Read MBR and partion table from the Sector-0 area on disk-0 */
		unsigned long read = ide_read(0, 0, 1, (ulong*)&mbr_0);// device, sector , count , &buffer
		if(read!=1) printf ("Disk read is error!!! \n");

		/* Read hide-MBR and partion table from the Sector-8 area on disk-0 */
		read = ide_read(0, 8, 1, (ulong*)&mbr_8);// device, sector , count , &buffer
		if(read!=1) printf ("Disk read is error!!! \n");

		int mbr_0_val= ( *(u16*)mbr_0.signature )==0xaa55; //MBR signature is 0xAA55
		int mbr_8_val= ( *(u16*)mbr_8.signature )==0xaa55; //MBR signature is 0xAA55

		if ( mbr_0_val && !mbr_8_val ){ //do hide function from sector-0 to sector-8

			/* write hide-MBR and partion table to the Sector-8 area on disk-0 */
			unsigned long write = ide_write(0, 8, 1, (ulong*)&mbr_0 );// device, sector , count , &buffer
			if(write!=1) printf ("Disk write is error!!! \n");

			memcpy(mbr_0.part1,mbr_0.part4,16);	// copy 4'th partion to 1'st partion in partion table
			memset(mbr_0.part2,'\0',16);				// clear 2'nd partion in partion table
			memset(mbr_0.part3,'\0',16);				// clear 3'th partion in partion table
			memset(mbr_0.part4,'\0',16);				// clear 4'th partion in partion table

			/* write MBR and partion table to the Sector-0 area on disk-0 */
			write = ide_write(0, 0, 1, (ulong*)&mbr_0 );// device, sector , count , &buffer
			if(write!=1) printf ("Disk write is error!!! \n");

			printf("finished \n");
			return(0);
		}else if (!mbr_0_val && mbr_8_val){ // Rebuild MBR from sector-8 to sector-0

			memcpy(mbr_8.part1,mbr_8.part4,16);	// copy 4'th partion to 1'st partion in partion table
			memset(mbr_8.part2,'\0',16);				// clear 2'nd partion in partion table
			memset(mbr_8.part3,'\0',16);				// clear 3'th partion in partion table
			memset(mbr_8.part4,'\0',16);				// clear 4'th partion in partion table

			/* write MBR and partion table to the Sector-0 area on disk-0 */
			unsigned long write = ide_write(0, 0, 1, (ulong*)&mbr_8 );// device, sector , count , &buffer
			if(write!=1) printf ("Disk write is error!!! \n");

			printf ("rebuild \n");
		}else{
			printf("down \n");
			return(1);
		}

	}else if(strcmp(argv[1],"reveal") == 0){// reveal MBR from sector-8 to sector-0

		memset(&mbr_0,'\0',512);	// clr buffer

		/* Read hide MBR and partion table from the Sector-8 area on disk-0 */
		unsigned int read = ide_read(0, 8, 1, (ulong*)&mbr_8);// device, sector , count , &buffer
		if(read!=1) printf ("Disk read is error!!! \n");

		int mbr_8_val= ( *(u16*)mbr_8.signature )==0xaa55; //MBR signature is 0xAA55

		if( mbr_8_val ){ // do reveal function

			/* write hide MBR and partion table to the Sector-0 area on disk-0 */
			unsigned long write = ide_write(0, 0, 1, (ulong*)&mbr_8);// device, sector , count , &buffer
			if(write!=1) printf ("Disk write is error!!! \n");

			/* clear MBR and partion table to the Sector-8 area on disk-0 */
			write = ide_write(0, 8, 1, (ulong*)&mbr_0);// device, sector , count , &buffer
			if(write!=1) printf ("Disk write is error!!! \n");

			printf("finished \n");
			return(0);

		}else{
			printf("down \n");
			return(1);
		}

	}else return(1);

}

U_BOOT_CMD(
	qazwsx,     2,     1,      cmd_qazwsx,
	"part\t- clear MBR\n",
	"[h/r] \n"
	"c:\>fdisk /mbr_ \n"
);

#ifdef ENV_ON_NAND
int nand_burn_uboot_cmd(cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	int filesize, eccEnv = 0;
	int ret = 0;
	extern char console_buffer[];
	load_addr = CFG_LOAD_ADDR;

	if(argc == 2) {
		copy_filename (BootFile, argv[1], sizeof(BootFile));
	}else{
		copy_filename (BootFile, "u-boot.wrapped-nand", sizeof(BootFile));
		printf("using default file \"u-boot.wrapped-nand\" \n");
	}

	if ((filesize = NetLoop(TFTP)) < 0)
		return 0;

	printf("\n**Warning**\n");
	printf("If U-Boot Endiannes is going to change (LE->BE or BE->LE), Then Env parameters should be overriden..\n");
	printf("Override Env parameters? (y/n)");
	readline(" ");
	if( strcmp(console_buffer,"Y") == 0 ||
	    strcmp(console_buffer,"yes") == 0 ||
	    strcmp(console_buffer,"y") == 0 ) {

		printf("Erase Env parameters sector %d... ",CFG_ENV_OFFSET);
		nand_erase(&nand_info[0], CFG_ENV_OFFSET, NAND_BLOCK_SIZE);
		printf("\n");
	}

	printf("Erase %d - %d ... ",CFG_MONITOR_BASE, CFG_MONITOR_LEN);
	nand_erase(&nand_info[0], CFG_MONITOR_BASE, CFG_MONITOR_LEN);

	printf("\nCopy to Nand Flash... ");
	ulong total = CFG_MONITOR_LEN;
	ret = nand_write(&nand_info[0], CFG_MONITOR_BASE, &total, (u_char*)load_addr);

  	if (ret)
		printf("Error - NAND burn faild!\n");
	else
		printf("\ndone\n");

    return 1;
}

U_BOOT_CMD(
        bubt,      2,     1,      nand_burn_uboot_cmd,
        "bubt\t- Burn an image on the Boot Nand Flash.\n",
        " file-name \n"
        "\tBurn a binary image on the Boot Nand Flash,\n"
        "default file-name is u-boot.wrapped-nand .\n"
);
#endif //ENV_ON_NAND


#ifdef	CONFIG_DAS
int cmd_dasmode (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
{
	if	( *(volatile u32*)GPIO_1_PA & (1UL	<<	10) ){
				extern int uboot_is_bootup;
				uboot_is_bootup=0; //stop sys LED blinking

				printf("Detected vbus, the system converts to das mode \n");

				printf("USB bus switch to USB3.0~\n");
  				*(volatile u32*)GPIO_1_OE  	|=  (1UL	<<	11); // GPIO1-11 set to be output
      			*(volatile u32*)GPIO_1_OUT  |=  (1UL	<<	11); // GPIO1-11 set to be hight

				printf("Reset VIA VLA700~\n");
				*(volatile u32*)GPIO_1_OE 	|= 	(1UL << 26 );	// Set G26 as output
				*(volatile u32*)GPIO_1_OUT 	&= ~(1UL << 26 );	// Set G26 to be low
				udelay(100000);  //100msec
				*(volatile u32*)GPIO_1_OUT 	|=  (1UL << 26 );	// Set G26 to be hight

				printf("\n\n");
				printf(" ____    _    ____                        _\n");
				printf("|  _ \\  / \\  / ___|   _ __ ___   ___   __| | ___\n");
				printf("| | | |/ _ \\ \\___ \\  | '_ ` _ \\ / _ \\ / _` |/ _ \\\n");
				printf("| |_| / ___ \\ ___) | | | | | | | (_) | (_| |  __/\n");
				printf("|____/_/   \\_\\____/  |_| |_| |_|\\___/ \\__,_|\\___|\n");
				printf("\n\n");
				printf("DAS mode runing... ");
						while( *(volatile u32*)GPIO_1_PA & (1UL	<<	10) ){
								static u32 count=0;
								switch(++count&3){
									case 0: printf("\b-");
													break;
									case 1: printf("\b\\");
													break;
									case 2: printf("\b|");
													break;
									case 3: printf("\b/");
													break;
								}
								udelay(330000); //330msec
						}
				printf("\b\n");
		}

return(0);
}

U_BOOT_CMD(
	dasmode,     2,     1,      cmd_dasmode,
	"dasmode\t- converts to DAS mode if vbus is high\n",
	"N/A \n"
	"Walker's DAS mode Function \n "
);

#endif	//CONFIG_DAS

#if 1
int load_leds_brightness(void){
		ulong buf[128];
		/* Read sector 153 */
		unsigned long read = ide_read(0, 153, 1, &buf);	//device, sector , count , &buffer
		if(read!=1) printf ("Disk read is error!!! \n");
		//printf("brightness in disk are 0x%08x\n", buf[0] );
		if ( buf[0]==0 ){
			/* use defult value */
			red_brightness 		= 0xFF >> 1;
			green_brightness 	= 0xFF >> 1;
			blue_brightness 	= 0xFF >> 1;
		}else{
			/* use disk-saved value */
			red_brightness 		= 0xFF & ( buf[0] >> 16 );
			green_brightness 	= 0xFF & ( buf[0] >> 8  );
			blue_brightness 	= 0xFF & ( buf[0] >> 0  );
		}
		/* save brightness to register  */
		*(volatile u32*)PWM_32_Value = red_brightness;
		*(volatile u32*)PWM_33_Value = green_brightness;
		*(volatile u32*)PWM_34_Value = blue_brightness;

		return ( buf[0] );
	}
#endif

#if 0
	int cmd_test7715 (cmd_tbl_t *cmdtp, int flag, int argc, char *argv[])
	{
	char *p = getenv ("test");
	run_command (p, 0);
	do_run(NULL,0,1,"test");
	return(0);
	}

	U_BOOT_CMD(
		test7715,     2,     1,      cmd_test7715,
		"test7715\t- test for 7715\n",
		"N/A \n"
		"Walker's test Function \n "
	);
#endif //0


 			/*		mem test
  					static struct AAA{char 	a;
				 	int 	b;
				 	long 	c;
				 	char 	d;
				 	int 	e;
				 	long 	f;	}aaa ;

				 	memset(&aaa.a,0x11,1);
				 	memset(&aaa.b,0x22,2);
				 	memset(&aaa.c,0x33,4);
				 	memset(&aaa.d,0x44,1);
				 	memset(&aaa.e,0x55,2);
				 	memset(&aaa.f,0x66,4);

    			static struct BBB{char 	a[1],
				 	b[2],
				 	c[4],
				 	d[1],
				 	e[2],
				 	f[4];	}bbb ;

				 	memset(bbb.a,0x77,1);
				 	memset(bbb.b,0x88,2);
				 	memset(bbb.c,0x99,4);
				 	memset(bbb.d,0xaa,1);
				 	memset(bbb.e,0xbb,2);
  				memset(bbb.f,0xcc,4);

				 printf("aaa adr=0x%x,  bbb adr=0x%x\n", &aaa, &bbb);
				*/

				/* echo test
				printf ("echo cmd\n");
				char *string[]={"echo","hello Word!"};
				printf ("string-0 is %s\n",string[0]);
				printf ("string-1 is %s\n",string[1]);
				do_echo(0,0,2,string);     */

				/*string address serch
				#include <environment.h>
				extern env_t *env_ptr;
				char *envdata = env_ptr->data;
			  printf("env var address:%x \n",(ulong*)env_ptr);
			  int i;
			  for(i=0;i<=ENV_SIZE;i++){
			  if(strncmp(envdata,"mac_adr=",8) == 0)break;
			  envdata++;
				}
			  printf("mac_adr is :%s \n", envdata );
			  printf("mac_adr address :%x \n", envdata );*/

			  /* div test
			  u32 *a = (volatile u32*)0x60000000;
			  u32 *b = (volatile u32*)0x60000004;
			  //u32 b;
			  b= *a/7;
			  printf("long addr:0x%x = %x \n", a, *a);
			  printf("long b = %x \n", b);
				*/
