/*
 * (C) Copyright 20090907 
 * UI Menu 
 * Jason_Chou Software Enginear Maintain
 *
 * General Desription:
 * Design ui menu function.This program lets users simple operate in u-boot.
 */
#include <common.h>
#include <command.h>
#include <linux/ctype.h>
#include <hush.h>
#include <net.h>
#include <version.h>
//#include <malloc.h>
//#include <devices.h>
#include <ui.h>

extern int reset_cpu(void);

void ui_reset(int arg);
void ui_load_kernel(int arg);
void ui_show_same_char(char c, int size);
void ui_show_boot_menu(void);
u32 char2decimal(char c);
u32 char2hex(char c);
int ui_gets(char *buf, int size);
void ui_ft_board_test(int arg, int *retValue);
int bt_FanLEDsBuzzer_test (void);
int bt_FanLEDsBuzzer_test (void);
void bt_NG(void);
void bt_OK(void);

int speed[]={183};

MENU_T boot_menu_table[]=
{
	{'0', NULL		,				0,	"Enter Command Line Interface"},
	{'1', (void *)ui_reset, 				0,	"Reboot"},
	{'2', (void *)ui_load_kernel,		0,	"Start the Kernel Code"},	
	{'T', (void *)ui_ft_board_test, 			0,	"GPIO Test Function"},
	{0,   NULL, 						0,	NULL},
};

/*----------------------------------------------------------------------
* ui_menu
*----------------------------------------------------------------------*/
void ui_menu(void)
{
	char command;
	int show_menu;
	MENU_T *menu;

	while (1)
	{
		ui_show_boot_menu();	
		command = getc();		
		command = toupper(command);
		
		if (command <= 0x20 || command >= 0x7f)
			continue;

		if(command == 0x30)
		{
			printf("\n");
			break;
		}
		
		printf("%c\n\n", command);
		
		menu = (MENU_T *)&boot_menu_table[0];
		while (menu->key)
		{
			if (menu->key == command)
			{
				menu->handler(menu->arg);
				break;
			}
			menu++;
		}
	}
}

#define MENU_TOTAL_ITEMS	(sizeof(boot_menu_table)/sizeof(MENU_T))
/*----------------------------------------------------------------------
* ui_show_boot_menu
*----------------------------------------------------------------------*/
void ui_show_boot_menu(void)
{
	MENU_T *menu;
	int count;
		
	printf("\n");
	ui_show_same_char(' ', 30);
	printf("Boot Menu\n");
	ui_show_same_char('=', 78);
	printf("\n");
	
	menu = (MENU_T *)&boot_menu_table[0];
	count = 0;
       
	while (menu->key)
	{
		printf("%c: ", menu->key);
		printf(menu->msg);
		if (MENU_TOTAL_ITEMS < 2)
		{
			printf("\n");	
		}
		else
		{
			if (count & 1)
				printf("\n");
			else
				ui_show_same_char(' ', 41-strlen(menu->msg));
		}
		count++;
		menu++;	
	}
	if (count & 1)
		printf("\n");
	printf("\n");
	printf("=> Select: ");
	
}


void ui_reset (int arg)
{
   reset_cpu();
}

/*----------------------------------------------------------------------
* ui_show_same_char(char data, int size)
*----------------------------------------------------------------------*/
void ui_show_same_char(char c, int size)
{
	char data[81], *cp;
	
	if (size > 80) 
		size = 80;
		
	cp = data;
	while (size--)
		*cp++ = c;
	*cp = 0x00;
	printf(data);
}

/*----------------------------------------------------------------------
* ui_load_kernel
*----------------------------------------------------------------------*/
void ui_load_kernel(int arg)
{
  char *s;

  s = getenv ("bootcmd");  
  run_command(s, 0);
}

/*----------------------------------------------------------------------
* char2decimal
*----------------------------------------------------------------------*/

u32 char2decimal(char c)
{
	if (c >= '0' && c <= '9')
		return (c - '0');	
	else 
		return (0xffffffff);
}

/*----------------------------------------------------------------------
* char2hex
*----------------------------------------------------------------------*/
u32 char2hex(char c)
{
	if (c >= '0' && c <= '9')
		return (c - '0');
	else if (c >= 'a' && c <= 'f')
		return (c - 'a' + 10);
	else if (c >= 'A' && c <= 'F')
		return (c - 'A' + 10);
	else
		return (0x100);
}

/*--------------------------------------------------------------
* 	uart_gets
---------------------------------------------------------------*/
int ui_gets(char *buf, int size)
{
	char key;
	int total;
	char *dest;
	int first_key = 1;
	
	if (size <= 0)
		return 0;
		
	total = strlen(buf);
	if (total >= size)
	{
		total = 0;
		dest = buf;
	}
	else
	{
		dest = buf + total;
		printf(buf);
	}
		
	
	do {
		key = getc();
		if (key == 0x0d || key == 0x0a)
			break;
		if (key == 0x03 || key == 0x1b)
		{
			printf("\n");
			return 0;
		}
		else if (key == '\b' || key == ARROW_L)
		{
			first_key = 0;
			if (total)
			{
				dest--;
				total--;
				putc('\b');
				putc(' ');
				putc('\b');
			}
		}
		else if (key >= 0x20 && key <= 0x7f)
		{
			if (first_key && total)
			{
				memset(buf, '\b', total); buf[total]=0x00;
				printf(buf);
				memset(buf, ' ', total);
				printf(buf);
				memset(buf, '\b', total);
				printf(buf);
				total = 0;
				dest = buf;
			}
			first_key = 0;
				
			if (total < size)
			{
				putc(key);
				*dest++ = key;
				total++;
			}
		}
	} while(1);
	
	*dest = 0x00;
	printf("\n");
	return total;
}


void ui_ft_board_test(int arg, int *retValue)
{
	ulong tmp1,tmp2;

	tmp1 = *(volatile u32*)GPIO_1_OUT;
	tmp2 = *(volatile u32*)GPIO_2_OUT;
		
	disable_interrupts();

	if (bt_FanLEDsBuzzer_test())
	{	
		bt_NG();
		 *(volatile u32*)GPIO_1_OUT = tmp1;
		 *(volatile u32*)GPIO_2_OUT = tmp2 ;
		enable_interrupts();
		return;	
	}
	if(bt_Button_test())
	{
		bt_NG();
		 *(volatile u32*)GPIO_1_OUT = tmp1;
		 *(volatile u32*)GPIO_2_OUT = tmp2 ;
		enable_interrupts();
		return;	
	}
	
	bt_OK();
	 *(volatile u32*)GPIO_1_OUT = tmp1;
	 *(volatile u32*)GPIO_2_OUT = tmp2 ;

	enable_interrupts();
}

int bt_FanLEDsBuzzer_test (void)
{
	char ans=0;
	printf("============================== \n");	
	printf("LED  test			  \n");	
	printf("============================== \n");	
	*(volatile u32*)GPIO_1_OUT &= ~ ( 1 << LED_HDDA_B |1 << FAN_PWM );					// Set GPIOA to be low	
	*(volatile u32*)GPIO_2_OUT &= ~( 1 << LED_HDDB_B |1 << LED_HDDA_R |1 << LED_HDDB_R \
								|1 << LED_STATUS_R |1 << LED_USB  | 1 << LED_STATUS_B | 1 << BUZZER_PIN);	// Set GPIOB to be low	

	printf("============================== \n");	
	printf("Is Blue LED Blinking ? (Y/N)\n");
	printf("============================== \n");	
	
	while((ans != 0x59) && (ans != 0x79) && (ans != 0x4e) && (ans != 0x6e))
	{
		*(volatile u32*)GPIO_1_OUT |= ( 1 << LED_HDDA_B);	// Set GPIOA to be High
		*(volatile u32*)GPIO_2_OUT |= ( 1 << LED_HDDB_B |1 << LED_USB  | 1 << LED_STATUS_B );	// Set GPIOB to be High
		udelay(500*1000);		//delay 500 msec
		
		*(volatile u32*)GPIO_1_OUT &= ~ ( 1 << LED_HDDA_B);					// Set GPIOA to be low	
		*(volatile u32*)GPIO_2_OUT &= ~( 1 << LED_HDDB_B |1 << LED_USB  | 1 << LED_STATUS_B );	// Set GPIOB to be low	
		udelay(500*1000);		//delay 500 msec
		
		if (tstc()) 
			ans=getc();
	}
					
	if( ans == 0x4e ||ans == 0x6e)
		return 1;

	udelay(50*1000);		//delay 50msec
	
	ans=0;	
	printf("============================== \n");	
	printf("Is Red LED Blinking ? (Y/N)\n");
	printf("============================== \n");	
	
	while((ans != 0x59) && (ans != 0x79) && (ans != 0x4e) && (ans != 0x6e))
	{
		*(volatile u32*)GPIO_2_OUT |= ( 1 << LED_HDDA_R |1 << LED_HDDB_R |1 << LED_STATUS_R);	// Set GPIOB to be high
		udelay(500*1000);		//delay 500 msec

		*(volatile u32*)GPIO_2_OUT &= ~( 1 << LED_HDDA_R |1 << LED_HDDB_R |1 << LED_STATUS_R);	// Set GPIOB to be low
		udelay(500*1000);		//delay 500 msec
		
		if (tstc()) 
			ans=getc();
	}

	if( ans == 0x4e ||ans == 0x6e)
		return 1;

	udelay(50*1000);		//delay 50msec
	
	ans=0;	
	printf("============================== \n");	
	printf("Is 'Bee' sound on and Fan turn on ? (Y/N)\n");
	printf("============================== \n");	
	
	*(volatile u32*)GPIO_1_OUT |= ( 1 << FAN_PWM );
	while((ans != 0x59) && (ans != 0x79) && (ans != 0x4e) && (ans != 0x6e))
	{		
		*(volatile u32*)PWM_43_Value = 0x7F;		// GPIO1-24 buzzer_pwm_duty set to 50%
		udelay(500000);		//delay 100msec
		*(volatile u32*)PWM_43_Value = 0x00;		// GPIO1-24 buzzer_pwm_duty set to 0%
		udelay(500000);		//delay 100msec		
		if (tstc()) 
			ans=getc();
	}

	if( ans == 0x4e ||ans == 0x6e)
		return 1;
	
	return 0;		
}

int bt_Button_test (void)
{
	ulong now_time;

	printf("============================== \n");	
	printf("Please press RESET button during 10 seconds...\n");
	printf("============================== \n");	
	now_time = get_timer_masked (); 
	while((*(volatile u32*)GPIO_1_PA & ( 1 << RESET_BTN ))  != 0)
	{
		if(get_timer_masked () > (now_time+10*CFG_HZ))
		{	
			printf("Timeout!\n");
			return 1;
		}
	}
	
	udelay(1000*1000);		//delay 1sec

	printf("============================== \n");		
	printf("Please press POWER button during 10 seconds...\n");
	printf("============================== \n");	
	now_time = get_timer_masked (); 
	while((*(volatile u32*)GPIO_1_PA & ( 1 << PWR_SW_BTN))  == 0)
	{
		if(get_timer_masked () > (now_time+10*CFG_HZ))
		{	
			printf("Timeout!\n");
			return 1;
		}
	}

	udelay(1000*1000);		//delay 1sec	
	
	printf("============================== \n");	
	printf("Please press HDDA button during 10 seconds...\n");
	printf("============================== \n");	
	now_time = get_timer_masked (); 
	while((*(volatile u32*)GPIO_1_PA & ( 1 << HDDA_BTN))  != 0)
	{
		if(get_timer_masked () > (now_time+10*CFG_HZ))
		{	
			printf("Timeout!\n");
			return 1;
		}
	}

	udelay(1000*1000);		//delay 1sec
	
	printf("============================== \n");	
	printf("Please press HDDB button during 10 seconds...\n");
	printf("============================== \n");	
	now_time = get_timer_masked (); 
	while((*(volatile u32*)GPIO_1_PA & ( 1 << HDDB_BTN ))  != 0)
	{
		if(get_timer_masked () > (now_time+10*CFG_HZ))
		{	
			printf("Timeout!\n");
			return 1;
		}
	}
	
	return 0;
}

void bt_NG(void)
{
	printf("\n======================================\n");
	printf("                XX       XX   \n");
	printf("                 XX     XX    \n");
	printf("                  XX   XX     \n");
	printf("                   XX XX      \n");
	printf("                    XXX       \n");
	printf("                   XX XX      \n");
	printf("                  XX   XX     \n");
	printf("                 XX     XX    \n");
	printf("                XX       XX   \n");
	printf("\n======================================\n");
}

void bt_OK(void)
{
	printf("\n======================================\n\n");
	printf("        OKOK        OK      OK \n");
	printf("      OK    OK	    OK     OK \n");
	printf("    OK        OK    OK    OK \n");
	printf("   OK          OK   OK   OK	\n");
	printf("  OK            OK  OK  OK \n");
	printf("  OK            OK  OK OK \n");
	printf("  OK            OK  OKOK \n");
	printf("  OK            OK  OK	OK \n");
	printf("  OK            OK  OK   OK \n");
	printf("   OK          OK   OK    OK \n");
	printf("    OK        OK    OK     OK \n");
	printf("      OK    OK      OK      OK \n");
	printf("        OKOK        OK       OK \n");
	printf("\n======================================\n");
}


